@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2015 Daniel Scharrer
+ * Copyright (C) 2011-2016 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -120,8 +120,12 @@ struct file_output {
 	util::ofstream stream;
 	
 	explicit file_output(const fs::path & file) : name(file) {
-		stream.open(name, std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
-		if(!stream.is_open()) {
+		try {
+			stream.open(name, std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
+			if(!stream.is_open()) {
+				throw 0;
+			}
+		} catch(...) {
 			throw std::runtime_error("Coul not open output file \"" + name.string() + '"');
 		}
 	}
@@ -388,37 +392,50 @@ static bool insert_dirs(DirectoriesMap & processed_directories, const path_filte
 	return false;
 }
 
-static void rename_collision(const extract_options & o, FilesMap & processed_files,
+static bool rename_collision(const extract_options & o, FilesMap & processed_files,
                              const std::string & path, const processed_file & other,
-                             bool common_component, bool common_language) {
+                             bool common_component, bool common_language, bool first) {
 	
 	const setup::file_entry & file = other.entry();
 	
+	bool require_number_suffix = !first || (o.collisions == RenameAllCollisions);
 	std::ostringstream oss;
 	
 	if(!common_component && !file.components.empty()) {
 		if(setup::is_simple_expression(file.components)) {
+			require_number_suffix = false;
 			oss << '#' << file.components;
 		}
 	}
-	if(!common_language && !file.languages.empty() && file.languages != o.default_language) {
+	if(!common_language && !file.languages.empty()) {
 		if(setup::is_simple_expression(file.languages)) {
-			oss << '@' << file.languages;
+			require_number_suffix = false;
+			if(file.languages != o.default_language) {
+				oss << '@' << file.languages;
+			}
 		}
 	}
 	
 	size_t i = 0;
 	std::string suffix = oss.str();
-	if(suffix.empty() && file.languages != o.default_language) {
+	if(require_number_suffix) {
 		oss << '$' << i++;
 	}
-	while(processed_files.find(path + oss.str()) != processed_files.end()) {
+	for(;;) {
+		std::pair<FilesMap::iterator, bool> insertion = processed_files.insert(std::make_pair(
+			path + oss.str(), processed_file(&file, other.path() + oss.str())
+		));
+		if(insertion.second) {
+			// Found an available name and inserted
+			return true;
+		}
+		if(&insertion.first->second.entry() == &file) {
+			// File already has the desired name, abort
+			return false;
+		}
 		oss.str(suffix);
 		oss << '$' << i++;
 	}
-	processed_files.insert(std::make_pair(
-		path + oss.str(), processed_file(&file, other.path() + oss.str())
-	));
 	
 }
 
@@ -439,13 +456,15 @@ static void rename_collisions(const extract_options & o, FilesMap & processed_fi
 			common_language = common_language && other.entry().languages == file.languages;
 		}
 		
-		if(common_component && !common_language && file.languages != o.default_language) {
-			rename_collision(o, processed_files, path, base, common_component, common_language);
+		bool ignore_component = common_component || o.collisions != RenameAllCollisions;
+		if(rename_collision(o, processed_files, path, base,
+		                    ignore_component, common_language, true)) {
 			processed_files.erase(path);
 		}
 		
 		BOOST_FOREACH(const processed_file & other, collision.second) {
-			rename_collision(o, processed_files, path, other, common_component, common_language);
+			rename_collision(o, processed_files, path, other,
+			                 common_component, common_language, false);
 		}
 		
 	}
@@ -466,8 +485,13 @@ void process_file(const fs::path & file, const extract_options & o) {
 		throw std::runtime_error("Input file \"" + file.string() + "\" is a directory!");
 	}
 	
-	util::ifstream ifs(file, std::ios_base::in | std::ios_base::binary);
-	if(!ifs.is_open()) {
+	util::ifstream ifs;
+	try {
+		ifs.open(file, std::ios_base::in | std::ios_base::binary);
+		if(!ifs.is_open()) {
+			throw 0;
+		}
+	} catch(...) {
 		throw std::runtime_error("Could not open file \"" + file.string() + '"');
 	}
 	
@@ -685,7 +709,7 @@ void process_file(const fs::path & file, const extract_options & o) {
 			
 			if(o.collisions == ErrorOnCollisions) {
 				throw std::runtime_error("Collision: " + path);
-			} else if(o.collisions == RenameCollisions) {
+			} else if(o.collisions == RenameAllCollisions) {
 				collisions[internal_path].push_back(processed_file(&file, path));
 			} else {
 				
@@ -703,7 +727,11 @@ void process_file(const fs::path & file, const extract_options & o) {
 					}
 				}
 				
-				if(!o.silent) {
+				if(o.collisions == RenameCollisions) {
+					const setup::file_entry & clobberedfile = skip ? file : existing.entry();
+					const std::string & clobberedpath = skip ? path : existing.path();
+					collisions[internal_path].push_back(processed_file(&clobberedfile, clobberedpath));
+				} else if(!o.silent) {
 					std::cout << " - ";
 					const std::string & clobberedpath = skip ? path : existing.path();
 					std::cout << '"' << color::dim_yellow << clobberedpath << color::reset << '"';
@@ -728,7 +756,7 @@ void process_file(const fs::path & file, const extract_options & o) {
 		
 	}
 	
-	if(o.collisions == RenameCollisions) {
+	if(o.collisions == RenameCollisions || o.collisions == RenameAllCollisions) {
 		rename_collisions(o, processed_files, collisions);
 		collisions.clear();
 	}
@@ -848,7 +876,7 @@ void process_file(const fs::path & file, const extract_options & o) {
 			// Print filename and size
 			if(o.list) {
 				
-				extract_progress.clear();
+				extract_progress.clear(DeferredClear);
 				
 				if(!o.silent) {
 					
