@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2015 Daniel Scharrer
+ * Copyright (C) 2011-2016 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -34,6 +34,12 @@ namespace setup {
 
 namespace {
 
+STORED_ENUM_MAP(stored_alpha_format, header::AlphaIgnored,
+	header::AlphaIgnored,
+	header::AlphaDefined,
+	header::AlphaPremultiplied
+);
+
 STORED_ENUM_MAP(stored_install_verbosity, header::NormalInstallMode,
 	header::NormalInstallMode,
 	header::SilentInstallMode,
@@ -298,13 +304,23 @@ void header::load(std::istream & is, const version & version) {
 	} else {
 		back_color2 = 0;
 	}
-	image_back_color = util::load<boost::uint32_t>(is);
+	if(version < INNO_VERSION(5, 5, 7)) {
+		image_back_color = util::load<boost::uint32_t>(is);
+	} else {
+		image_back_color = 0;
+	}
 	if(version >= INNO_VERSION(2, 0, 0) && version < INNO_VERSION(5, 0, 4)) {
 		small_image_back_color = util::load<boost::uint32_t>(is);
 	} else {
 		small_image_back_color = 0;
 	}
 	
+	if(version >= INNO_VERSION(5, 5, 7)) {
+		image_alpha_format = stored_enum<stored_alpha_format>(is).get();
+	} else {
+		image_alpha_format = AlphaIgnored;
+	}
+	
 	if(version < INNO_VERSION(4, 2, 0)) {
 		password.crc32 = util::load<boost::uint32_t>(is);
 		password.type = crypto::CRC32;
@@ -558,6 +574,9 @@ void header::load(std::istream & is, const version & version) {
 	} else {
 		options |= AllowNetworkDrive;
 	}
+	if(version >= INNO_VERSION(5, 5, 7)) {
+		flagreader.add(ForceCloseApplications);
+	}
 	
 	options |= flagreader;
 	
@@ -652,6 +671,7 @@ NAMES(setup::header::flags, "Setup Option",
 	"close applications",
 	"restart applications",
 	"allow network drive",
+	"force close applications",
 	"uninstallable",
 	"disable dir page",
 	"disable program group page",
@@ -674,6 +694,12 @@ NAMES(setup::header::architecture_types, "Architecture",
 	"IA64",
 )
 
+NAMES(setup::header::alpha_format, "Alpha Format",
+	"ignored",
+	"defined",
+	"premultiplied",
+)
+
 NAMES(setup::header::install_verbosity, "Install Mode",
 	"normal",
 	"silent",
