@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Daniel Scharrer
+ * Copyright (C) 2011-2016 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -110,8 +110,8 @@ bool inno_lzma1_decompressor_impl::filter(const char * & begin_in, const char *
 		if(properties > (9 * 5 * 5)) {
 			throw lzma_error("inno lzma1 property error", LZMA_FORMAT_ERROR);
 		}
-		options.pb = properties / (9 * 5);
-		options.lp = (properties % (9 * 5)) / 9;
+		options.pb = boost::uint32_t(properties / (9 * 5));
+		options.lp = boost::uint32_t((properties % (9 * 5)) / 9);
 		options.lc = properties % 9;
 		
 		options.dict_size = util::little_endian::load<boost::uint32_t>(header + 1);
